/*
* This example code is used for LaskaKit microESP32-C3 board https://www.laskakit.cz/laskakit-microesp/
* with our Temperature and Humidity SHT40 module https://www.laskakit.cz/laskakit-sht40-senzor-teploty-a-vlhkosti-vzduchu/
* MicroESP32-C3 board reads temperature and humidity from SHT40 sensor 
* and sends every second through native USB
* 
* SDA - GPIO8
* SCL - GPIO10
*
* !!! ESP library version !!!
* ESP32 library 3.0.x
* Condition: Tools -> if USB CDC On Boot is ENABLED then
* Serial means native USB
* Tools -> if USB CDC On Boot is DISABLED then
* Serial means UART
* ---
* ESP32 library 2.0.xy
* Condition: Tools -> USB CDC On Boot must be disabled
* use USBSerial to send data through native USB 
*
* Board: ESP32-C3 Dev Module
*
* Libraries: https://github.com/Sensirion/arduino-i2c-sht4x
* 
*/

#include <Arduino.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SharpMem.h>
#include <SensirionI2cSen66.h>

#include "Roboto_Black_18.h"
#include "Roboto_Black_24.h"
#include "Roboto_Black_42.h"
#include "Roboto_Black_72.h"

// I2C
#define SDA 8
#define SCL 10

// SPI
#define SHARP_SCK  14
#define SHARP_MOSI 13
#define SHARP_CS   15

// uSup power pin
#define USUP_POWER 2 //uSUP connector power output pin

// Memory LCD
Adafruit_SharpMem display(SHARP_SCK, SHARP_MOSI, SHARP_CS, 400, 240);

#define BLACK 0
#define WHITE 1

SensirionI2cSen66 sensor;
static int16_t error;
#define NO_ERROR 0

void setup() {
  //WiFi.mode(WIFI_MODE_NULL); // disable Wi-Fi
  Serial.begin(115200);
  //while (!Serial) 
  {
    delay(100);
  }

  pinMode(USUP_POWER, OUTPUT);    // Set uSUP connector power transistor as output
  digitalWrite(USUP_POWER, HIGH); // Turn power ON uSUP connector
  delay(100);            // Delay so it has time to turn on

  //Wire.begin(SDA,SCL);
  Wire.begin();

  // memory LCD
  display.begin();  // initialization
  display.setRotation(1); // rotation
  display.clearDisplay(); // clear display
  display.setCursor(0,0); // set cursor
  display.setTextColor(BLACK, WHITE); // set color

  // sen66
  sensor.begin(Wire, SEN66_I2C_ADDR_6B);
  error = sensor.deviceReset();
  if (error != NO_ERROR) 
  {
    Serial.print("Error trying to execute deviceReset(): ");
    return;
  }

  error = sensor.startContinuousMeasurement();
    if (error != NO_ERROR) {
      Serial.print("Error trying to execute startContinuousMeasurement(): ");
      return;
    }

}

void loop() {
  float massConcentrationPm1p0 = 0.0;
  float massConcentrationPm2p5 = 0.0;
  float massConcentrationPm4p0 = 0.0;
  float massConcentrationPm10p0 = 0.0;
  float humidity = 0.0;
  float temperature = 0.0;
  float vocIndex = 0.0;
  float noxIndex = 0.0;
  uint16_t co2 = 0;
  delay(1000);

  error = sensor.readMeasuredValues(
        massConcentrationPm1p0, massConcentrationPm2p5, massConcentrationPm4p0,
        massConcentrationPm10p0, humidity, temperature, vocIndex, noxIndex,
        co2);

  Serial.print("massConcentrationPm1p0: ");
  Serial.print(massConcentrationPm1p0);
  Serial.print("\t");
  Serial.print("massConcentrationPm2p5: ");
  Serial.print(massConcentrationPm2p5);
  Serial.print("\t");
  Serial.print("massConcentrationPm4p0: ");
  Serial.print(massConcentrationPm4p0);
  Serial.print("\t");
  Serial.print("massConcentrationPm10p0: ");
  Serial.print(massConcentrationPm10p0);
  Serial.print("\t");
  Serial.print("humidity: ");
  Serial.print(humidity);
  Serial.print("\t");
  Serial.print("temperature: ");
  Serial.print(temperature);
  Serial.print("\t");
  Serial.print("vocIndex: ");
  Serial.print(vocIndex);
  Serial.print("\t");
  Serial.print("noxIndex: ");
  Serial.print(noxIndex);
  Serial.print("\t");
  Serial.print("co2: ");
  Serial.print(co2);
  Serial.println();

  // memory LCD
  display.setRotation(1); // rotation
  display.clearDisplay(); // clear display
  display.setTextColor(BLACK, WHITE); // set color

  display.setFont(&Roboto_Black_72);

  display.setCursor(10,82);
  display.println(co2);

  display.setCursor(10,82*2);
  display.println(String(temperature, 1));
  
  display.setCursor(10,82*3);
  display.println(String(humidity, 0));

  display.setCursor(10,82*4);
  display.println(String(massConcentrationPm4p0, 0));

  display.setFont(&Roboto_Black_42);

  display.setCursor(10,78*5);
  display.println(String(vocIndex, 0));

  display.setCursor(150,78*5);
  display.println(String(noxIndex, 0));

  display.setCursor(10, 82-60);
  display.setFont(&Roboto_Black_24);
  display.println("CO2");
  display.setCursor(170, 82-60);
  display.setFont(&Roboto_Black_18);
  display.println("ppm");

  display.setCursor(10, 82*2-60);
  display.setFont(&Roboto_Black_24);
  display.println("Teplota");
  display.setCursor(170, 82*2-60);
  display.setFont(&Roboto_Black_18);
  display.println("degC"); 

  display.setCursor(10, 82*3-60);
  display.setFont(&Roboto_Black_24);
  display.println("Vlhkost");
  display.setCursor(170, 82*3-60);
  display.setFont(&Roboto_Black_18);
  display.println("%Rh");

  display.setCursor(10, 82*4-60);
  display.setFont(&Roboto_Black_24);
  display.println("PM4");
  display.setCursor(160, 82*4-60);
  display.setFont(&Roboto_Black_18);
  display.println("ug/m3");

  display.setCursor(10, 82*5-55);
  display.setFont(&Roboto_Black_24);
  display.println("VOC");

  display.setCursor(150, 82*5-55);
  display.setFont(&Roboto_Black_24);
  display.println("NOx");

  display.refresh(); // update the screen
}